/*jshint loopfunc: true */
/*global
  $, colors
*/
"use strict";
var urlPollerCallbacks = {},
    urlPollerInterval = {};
// interval is only used for first callback per url
function addPoller(url, interval, callback) {
    var firstTime = false,
        refreshFunction = function () {
            $.ajax({
                url: url,
                cache: false,
                success: function (data) {
                    urlPollerCallbacks[url].forEach(function (callback) {
                        callback(data, firstTime);
                    });
                    firstTime = false;
                },
                dataType: "json"
            }).complete(function () {
                setTimeout(refreshFunction, interval);
            });
        };
    if (urlPollerCallbacks[url]) {
        urlPollerCallbacks[url].push(callback);
        if (interval < urlPollerInterval[url]) {
            //if poller for url already exist and new request for url is made the lowest interval will be used for all callbacks.
            urlPollerInterval[url] = interval;
            refreshFunction();
        }
    } else {
        urlPollerCallbacks[url] = [callback];
        urlPollerInterval[url] = [interval];
        firstTime = true;
        refreshFunction();
    }
}

(function () {
    $.widget("cognimatics.liveTable", {
        options: {
            url: "counts.json",
            resolution: 2,
            interval: 2000,
            columns: [{type: "in", title: "In"}, {type: "out", title: "Out"}],
            viewRefreshTimeout: 2000 //This is currently not used, it updates on ajax callback only.
        },
        _create: function () {
            var options = this.options,
                latest_stats = {},
                passes = [],
                forceUpdateTimeout,
                $e = this.element.html("").addClass("liveTable"),
                $head = $("<tr/>").addClass("time"),
                dataCellWidth = Math.floor(100 / (3 + options.columns.length)),
                $waiting,
                groupFormats,
                new_count = false;
            $("<thead/>").appendTo($e).append($head);
            $head.append($("<th/>").css("width", "60%"));
            options.columns.forEach(function (column) {
                $("<th/>").html(column.title).appendTo($head);
            });
            $waiting = $("<td/>")
                .html("Waiting for passages...")
                .attr("colspan", "3")
                .addClass("waiting time");
            $("<tr/>").append($waiting).appendTo($e);
            groupFormats = [
                { size: 1, stop: 10, singular: "%d second ago", plural: "%d seconds ago"},
                //As next group is a group of 10's of seconds it's always plural and number must be multiplied by 10.
                { size: 10, stop: 60, singular: "%d to %d2 seconds ago", plural: "%d to %d2 seconds ago", label_number_modifier: 10},
                { size: 60, stop: 60 * 10, singular: "%d minute ago", plural: "%d minutes ago"},
                //As next group is a group of 10's of minutes it's always plural and number must be multiplied by 10.
                { size: 60 * 10, stop: 60 * 60, singular: "%d to %d2 minutes ago", plural: "%d to %d2 minutes ago", label_number_modifier: 10},
                { size: 60 * 60, stop: 60 * 60 * 24, singular: "%d hour ago", plural: "%d hours ago"},
                { size: 60 * 60 * 24, stop: 60 * 60 * 24 * 7, singular: "%d day ago", plural: "%d days ago"},
                { size: 60 * 60 * 24 * 7, stop: Infinity, singular: "%d week ago", plural: "once upon a time"}
            ];

            function updateView(init_timeout) {
                //As ajax is run as often as every 2 seconds and we update on it's callback we do not need to set a separate interval for update.
                /*init_timeout = init_timeout || false;
                if (init_timeout) {
                    setInterval(updateView, options.viewRefreshTimeout);
                }*/
                var rows = [],
                    i,
                    row,
                    label_nbr,
                    label_nbr_high,
                    label,
                    $tr;

                passes.forEach(function (pass) {
                    var age = Date.now() / 1000 - pass.time,
                        groupFormat,
                        ageGroup,
                        latestRow,
                        newRow;
                    age = Math.floor(age / options.resolution) * options.resolution;
                    groupFormat = groupFormats.reduce(function (previous, current) {
                        return age < current.size ? previous : current;
                    });
                    ageGroup = Math.floor(age / groupFormat.size);

                    latestRow = rows.length && rows[rows.length - 1];
                    if (latestRow &&
                            latestRow.ageGroup !== ageGroup ||
                            latestRow.formatSize !== groupFormat.size) {
                        latestRow = false;
                    }

                    if (!latestRow) {
                        newRow = {
                            ageGroup: ageGroup,
                            formatSize: groupFormat.size,
                            data: {},
                            groupFormat: groupFormat
                        };
                        options.columns.forEach(function (column) {
                            newRow.data[column.type] = 0;
                        });
                        rows.push(newRow);
                        latestRow = rows[rows.length - 1];
                    }

                    latestRow.data[pass.type] += 1;
                });
                if (passes.length || !$e.find(".waiting").length) {
                    $e.find("tr").slice(1).remove();
                    for (i = rows.length - 1; i >= 0; i -= 1) {
                        row = rows[i];
                        label_nbr = row.ageGroup;
                        label_nbr_high = row.ageGroup + 1;
                        if (row.groupFormat.label_number_modifier !== undefined) {
                            label_nbr *= row.groupFormat.label_number_modifier;
                            label_nbr_high *= row.groupFormat.label_number_modifier;
                        }
                        label = row.groupFormat[row.ageGroup === 1 ? "singular" : "plural"].replace("%d", label_nbr).replace("%d2", label_nbr_high);
                        $tr = $("<tr/>");
                        if (new_count && i === rows.length - 1) {
                            $tr.css("background", colors.green20);
                            $tr.delay(500).animate({
                                backgroundColor: 'rgba(0,0,0,0)'
                            });
                            $tr.addClass("highlight");
                            new_count = false;
                        }
                        $tr.append($("<td/>").html(label).addClass("time"));
                        options.columns.forEach(function (column) {
                            $tr.append($("<td/>").html(row.data[column.type]));
                        });
                        $tr.appendTo($e);
                    }
                }
            }
            updateView(true);
            addPoller(options.url, options.interval, function (data) {
                //new_count = false;
                var t = parseInt(Date.now() / 1000),
                    i;
                new_count = false;
                if (latest_stats) {
                    options.columns.forEach(function (column) {
                        var diff = data[column.type] - latest_stats[column.type];
                        for (i = 0; i < diff; i++) {
                            passes.push({time: t, type: column.type});
                            new_count = true;
                        }
                    });
                }
                options.columns.forEach(function (column) {
                    latest_stats[column.type] = data[column.type];
                });
                updateView();
            });
        }
    });
    $.widget("cognimatics.todayTable", {
        options: {
            interval: 2000,
            show: [],
        },
        _create: function () {
            var options = this.options,
                $head = $("<tr/>"),
                $body = $("<tr/>"),
                i,
                show,
                html;
            for (i in this.options.show) {
                show = this.options.show[i];
                html = show.title;
                if (show.icon) {
                    html = show.title + "&nbsp;<i class=\"%\"></i>".replace("%", show.icon);
                }
                $("<th/>")
                    .html(html)
                    .addClass(show.titleClass)
                    .appendTo($head);
                show.element = $("<td/>")
                    .html("")
                    .appendTo($body);
            }
            $head.appendTo(this.element);
            $body.appendTo(this.element);
            this.element.addClass("todayTable");
            addPoller(options.url, options.interval, function (data) {
                options.show.forEach(function (show) {
                    var val = data[show.key];
                    show.element.html(val);
                });
            });
        },
        _fresh: function () {
        }
    });

    $.widget("cognimatics.statTable", {
        options: {
            url: undefined,
            interval: 600000,
            showAverage: true
        },
        _create: function () {
            var me = this,
                $table = me.element;
            $table.addClass("statTable");
            $table.css("border", "1px solid " + colors.gray20);
            me.element.fadeTo(0, 0.1);
            addPoller(me.options.url, me.options.interval, function (response, firstTime) {
                me.element.fadeTo(500, 1);
                me.element.html("");
                var $header = $("<tr/>");
                $("<th/>").appendTo($header);
                response.data.forEach(function (bin) {
                    $("<th/>")
                        .html(bin.datetime)
                        .appendTo($header);
                });
                if (me.options.showAverage) {
                    $("<th/>").html("Average").appendTo($header);
                }
                $table.append($header);
                response.meta.headers.forEach(function (dataType) {
                    if (dataType.sensor) {
                        var odd = false,
                            $tr = $("<tr/>"),
                            $td,
                            valsum = 0;
                        $("<td/>").html(dataType.title).appendTo($tr);
                        response.data.forEach(function (bin) {
                            var $td = $("<td/>"),
                                val = bin[dataType.keyword];
                            valsum += val;
                            $td.html(val).appendTo($tr);
                            if (odd) {
                                $td.css("background-color", me.options.theme.odd_cells);
                            }
                            odd = !odd;
                        });
                        if (me.options.showAverage) {
                            valsum /= response.data.length;
                            $td = $("<td/>");
                            $td.html(valsum | 0).appendTo($tr);
                            if (odd) {
                                $td.css("background-color", me.options.theme.odd_cells);
                            }
                        }
                        $tr.appendTo($table);
                    }
                });
            });
        }
    });
})();
