$(function() {
    var $startButton = $("#start-button"),
        $stopButton = $("#stop-button"),
        $progressBar = $("#progressbar"),
        $progressText = $("#progress-label"),
        $proceedButton = $("#proceed-button"),
        $stepText = $("#status-text"),
        $infoText = $("#info-text"),
        timer;
    $.ajaxSetup({ cache: false });
    $proceedButton.attr("formaction", location.protocol + "//" + location.host + "/stereo/");

    var processStarted = false;
    $startButton.click(function(e) {
        e.preventDefault();
        $stepText.html("Starting...");
        $progressText.text("");
        $progressBar.slideDown();
        $.post("/stereo/calibrate/start");
        $startButton.attr("disabled", "disabled");
        $proceedButton.attr("disabled", "disabled");
        clearInterval(timer);
        timer = setInterval(updateState, 2000);
        processStarted = true;
    });

    $stopButton.click(function(e) {
        e.preventDefault();
        $.post("/stereo/calibrate/stop");
        almostDone = false;
        clearInterval(timer);
        $stopButton.attr("disabled", "disabled");
        $stepText.html("Aborting... This might take a while.");
        if ($progressBar.css('display') === 'none') {
            $progressBar.show();
        }
        $progressBar.progressbar("value", false);
        timer = setInterval(updateState, 5000);
        processStarted = false;
    });

    var almostDone = false;

    $progressBar.progressbar({
        value: false,
        change: function() {
            var cur_val = $progressBar.progressbar( "value" );
            if (cur_val === false) {
                $progressText.text("");
            } else {
                $progressText.text(cur_val  + "%" );
            }
        },
      complete: function() {
        $progressText.text( "Complete!" );
      }
    }).hide();

    function updateUI() {
        $.getJSON("/stereo/calibration.json", function(calib) {
            if (params.device.calibration.has_calibration && calib.step === 0) {
                $proceedButton.removeAttr("disabled");
            } else {
                $proceedButton.attr("disabled", "disabled");
            }
            almostDone = almostDone || calib.step > 0;
            if ( calib.step === 0 ) {
                if (almostDone) {
                    $startButton.removeAttr("disabled");
                    $stopButton.attr("disabled", "disabled");
                    $stepText.html("Calibration completed!");
                    processStarted = false;
                } else if (!processStarted) {
                    $startButton.removeAttr("disabled");
                    $stopButton.attr("disabled", "disabled");
                    if ( params.device.calibration.has_calibration ) {
                        $stepText.html("Camera is calibrated - Ready for recalibration.");
                    } else {
                        $stepText.html("This camera is not calibrated - Ready for calibration.");
                    }
                }
                $progressBar.slideUp();
            } else if ( calib.step == 1 ) {
                if ($progressBar.css('display') === 'none') {
                    $progressBar.show();
                }
                $stopButton.removeAttr("disabled");
                $startButton.attr("disabled", "disabled");
                $stepText.html("Collecting video data... Please make sure there are people moving beneath the camera.");
                var progress = Math.min(50, calib.points / calib.target_points * 50);
                $progressBar.progressbar("value", Math.round(progress));
            } else if ( calib.step == 2 ){
                if ($progressBar.css('display') === 'none') {
                    $progressBar.show();
                }
                $progressBar.progressbar("value", false);
                $stopButton.removeAttr("disabled");
                $startButton.attr("disabled", "disabled");
                $stepText.html("Processing the collected video data...");
            } else if( calib.step == 3 ) {
                $progressBar.progressbar("value", false);
                $stopButton.removeAttr("disabled");
                $startButton.attr("disabled", "disabled");
                $stepText.html("Calibration complete, applying new calibration.");
            } else if( calib.step == 4 ) {
                // We will probably never come here since the camera restarts
                // and gets mode = "off" when complete.
                // That's why we have the variable almostDone.
            } else {
            }
        });
    }
    function updateState() {
        $.getJSON("/stereo/params.json", function(data) {
            params = data;
            updateUI();
        });

    }
    updateUI();
    timer = setInterval(updateState, 5000);
    $("#left .centered").createVideo();
});
